@@ -32,7 +32,7 @@
 from abc import ABCMeta, abstractmethod
 from typing import Dict, List, Optional, Set, Type
 
-from pygasus.model import Model, Sequence
+from pygasus.model import CustomField, Field, Model, Sequence
 from pygasus.storage.query_builder import AbstractQueryBuilder
 
 
@@ -51,8 +51,41 @@ class AbstractStorageEngine(metaclass=ABCMeta):
 
     def __init__(self):
         self.models = {}
+        self.custom_fields = {}
         self.cache = {}
 
+    def add_custom_field(self, field: Type[CustomField], **kwargs):
+        """Add support for a custom field.
+
+        Args:
+            field (CustomField): the custom field class.
+
+        Additional keyword arguments can be sent and will be stored
+        as options for the custom field.
+
+        """
+
+    def add_custom_field_to_model(
+        self,
+        custom_field: Type[CustomField],
+        model: Model,
+        field: Field,
+    ) -> CustomField:
+        """Add a custom field to a model.
+
+        Args:
+            custom_field (CustomField): the custom field to add.
+            model (Model): the model to add this field to.
+            field (Field): the generic field.
+
+        Returns:
+            custom_field (CustomField): the instanciated custom field.
+
+        """
+        custom_field = custom_field(self, model, field)
+        self.custom_fields[(model, field.name)] = custom_field
+        return custom_field.add()
+
     # Abstract methods
     @abstractmethod
     def init(self):
@@ -187,6 +220,19 @@ class AbstractStorageEngine(metaclass=ABCMeta):
 
         """
 
+    @abstractmethod
+    def bulk_delete(self, model: Type[Model], query) -> int:
+        """Select one or more models with the specified query and delete them.
+
+        Args:
+            model (subclass of Model): the model object.
+            query: the query object by which to filter.
+
+        Returns:
+            number (int): the number of deleted rows.
+
+        """
+
     # Helper methods.
     def get_back_field(self, model, field, left):
         """Return, if found, the bac field of the relationship.
