@@ -44,6 +44,7 @@ except ImportError:
 
 from pydantic import EmailStr
 from sqlalchemy import (
+    Boolean,
     Column,
     Date,
     DateTime,
@@ -65,6 +66,7 @@ from pygasus.storage.abc import AbstractStorageEngine
 from pygasus.storage.sql.query_builder import SQLQueryBuilder
 
 SQL_TYPES = {
+    bool: Boolean,
     bytes: LargeBinary,
     date: Date,
     datetime: DateTime,
@@ -190,6 +192,12 @@ class SQLStorageEngine(AbstractStorageEngine):
             f_type = field.type_
             info = field.field_info
             origin = get_origin(o_type)
+
+            # If it's a custom field, use that instead.
+            custom = info.extra.get("custom_class")
+            if custom:
+                f_type = self.add_custom_field_to_model(custom, model, field)
+
             if origin is list or o_type is Sequence[f_type]:
                 # Analyze the back field.  If it's a list too, an
                 # intermediate table should be created.
@@ -201,7 +209,7 @@ class SQLStorageEngine(AbstractStorageEngine):
                     right_field=self.get_back_field(model, field, f_type),
                 )
                 continue
-            elif issubclass(f_type, Model):
+            elif isinstance(f_type, type) and issubclass(f_type, Model):
                 # Create foreign key fields.
                 to_name = getattr(
                     f_type.__config__, "model_name", f_type.__name__.lower()
@@ -232,7 +240,7 @@ class SQLStorageEngine(AbstractStorageEngine):
                 continue
 
             # Handle enumerations here.
-            if issubclass(f_type, enum.Enum):
+            if isinstance(f_type, type) and issubclass(f_type, enum.Enum):
                 e_type = type(list(f_type)[0].value)
                 if not all(
                     isinstance(member.value, e_type) for member in f_type
@@ -343,6 +351,11 @@ class SQLStorageEngine(AbstractStorageEngine):
                 linked.add(name)
                 continue
 
+            custom = self.custom_fields.get((model, field.name))
+            if custom:
+                value = attrs[name]
+                sql[name] = custom.to_storage(value)
+
             if issubclass(f_type, Model):
                 right = attrs[name]
                 primary_keys = {
@@ -623,8 +636,7 @@ class SQLStorageEngine(AbstractStorageEngine):
         return objs
 
     def update(self, model, instance, key, old_value, new_value):
-        """
-        Update an instance attribute.
+        """Update an instance attribute.
 
         Args:
             model (subclass of Model): the model class.
@@ -646,6 +658,7 @@ class SQLStorageEngine(AbstractStorageEngine):
                 value = getattr(instance, name)
                 sql_primary_keys.append(getattr(sql_table.c, name) == value)
 
+        field = model.__fields__[key]
         # Handles enum field.
         if isinstance(new_value, enum.Enum):
             new_value = new_value.value
@@ -655,7 +668,12 @@ class SQLStorageEngine(AbstractStorageEngine):
             self, f"{type(new_value).__name__.lower()}_to_storage", None
         )
         if method:
-            new_value = method(model, model.__fields__[key], new_value)
+            new_value = method(model, field, new_value)
+
+        # Handle custom fields.
+        custom = self.custom_fields.get((model, field.name))
+        if custom:
+            new_value = custom.to_storage(new_value)
 
         # Send the query.
         sql_columns = {key: new_value}
@@ -701,6 +719,49 @@ class SQLStorageEngine(AbstractStorageEngine):
         delete = sql_table.delete().where(*sql_primary_keys)
         self.connection.execute(delete)
 
+    def bulk_delete(self, model: Type[Model], query) -> int:
+        """Select one or more models with the specified query and delete them.
+
+        Args:
+            model (subclass of Model): the model object.
+            query: the query object by which to filter.
+
+        Returns:
+            number (int): the number of deleted rows.
+
+        """
+        model_name = getattr(
+            model.__config__, "model_name", model.__name__.lower()
+        )
+        table = self.tables[model_name]
+        pks = []
+        for name, field in model.__fields__.items():
+            info = field.field_info
+            pk = info.extra.get("primary_key", False)
+            if pk:
+                pks.append(name)
+
+        sql = table.delete().where(query)
+
+        # Perform a select (this will be an additional query, but
+        # we need to safely invalidate the cache).
+        for obj in self.select(model, query):
+            specific_pks = []
+            for name in pks:
+                value = getattr(obj, name)
+                # Convert other field types.
+                method = getattr(
+                    self, f"{type(value).__name__.lower()}_to_storage", None
+                )
+                if method:
+                    value = method(model, field, value)
+                specific_pks.append(value)
+            self.cache.pop((model,) + tuple(specific_pks), None)
+
+        # Send the query.
+        ret = self.connection.execute(sql)
+        return ret.rowcount
+
     def _get_columns_for(self, model: Type[Model]) -> Tuple[Column]:
         """Return the relevant columns for the specified model.
 
@@ -764,6 +825,7 @@ class SQLStorageEngine(AbstractStorageEngine):
         done = done or ()
         objs = []
         done = list(done)
+        customs = []
         for model in models:
             done.append(model)
             model_name = getattr(
@@ -780,6 +842,7 @@ class SQLStorageEngine(AbstractStorageEngine):
                 f_type = field.type_
                 pk = info.extra.get("primary_key", False)
                 value = row.get(f"{model_name}_{name}", ...)
+                custom = self.custom_fields.get((model, field.name))
                 if pk:
                     if value is not ...:
                         pks.append(value)
@@ -794,6 +857,10 @@ class SQLStorageEngine(AbstractStorageEngine):
                     if method:
                         value = method(model, field, value)
 
+                    attrs[name] = value
+                elif custom:
+                    value = custom.to_field(value)
+                    customs.append((value, name))
                     attrs[name] = value
                 elif (
                     isinstance(f_type, type)
@@ -835,6 +902,9 @@ class SQLStorageEngine(AbstractStorageEngine):
                 objs.append(obj)
             else:
                 obj = model(**attrs)
+                for field, name in customs:
+                    field.parent = obj
+                    field.field = name
                 self.cache[(model,) + tuple(pks)] = obj
                 objs.append(obj)
                 for name, field in model.__fields__.items():
